
/* 
 * SuperCollider3 source file "SFAnker.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class SFAnker ----------------------------------------------------------
// 
SFAnker {

    // --- attributes

    var <>name; // type  String


    // --- relationships

    var <>time; // 0..1-relation to type  SFTime
    var <>relativeTo; // 0..1-relation to type  SFAnker
    var <>ownerSlice; // 0..1-relation to type  SFSlice



    // --- new(name, slice, relativeTo) : SFAnker -----------------------------
    //        
    *new { arg name, slice, relativeTo; // types String, SFSlice, SFAnker           
        ^super.new.init(name, slice, relativeTo); // type  SFAnker
    } // end new        


    // --- init(nam, slc, rel) : SFAnker --------------------------------------
    //        
    init { arg nam, slc, rel; // types String, SFSlice, SFAnker           
        name = nam;
        ownerSlice = slc;
        relativeTo = rel; // may be nil
        if ( slc != nil , {
            slc.ankersByName.put(name, this); // register at slice
        });
    } // end init        


    // --- relative(name) : SFAnker -------------------------------------------
    //      
    relative { arg name; // type String         
        ^SFAnker.new(name, this.ownerSlice, this); // type  SFAnker
    } // end relative        


    // --- post() : void ------------------------------------------------------
    //   
    post {        
        ("<"++name++":").post; time.post;
        if ( relativeTo != nil, {
            " (".post; relativeTo.post; ")".post;
        });
        ">".post;
    } // end post        


    // --- postln() : void ----------------------------------------------------
    //   
    postln {        
        this.post; "".postln;
    } // end postln        


} // end SFAnker
