
/* 
 * SuperCollider3 source file "SFCuelist.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class SFCuelist --------------------------------------------------------
// 
SFCuelist {

    // --- attributes

    var <>name; // type  String
    var <>loop = false; // type  boolean


    // --- relationships

    var <>cues; // 0..*-relation to type  SFSlice
    var <>masterslice; // 0..1-relation to type  SFSlice



    // --- new(masterslice) : SFCuelist ---------------------------------------
    //      
    *new { arg masterslice; // type SFSlice         
        ^super.new.masterslice_(masterslice).clear; // type  SFCuelist
    } // end new        


    // --- cue(slice, repeat) : SFCuelist -------------------------------------
    //
    // <p>Adds a slice reference to the cuelist. The slice to be added can be either identified by its name string, or directly be passed by its SFSlice instance.</p>
    // <p>For other operations on the cue list (remove, etc.), directly access the cue list via cl.cues.dothisandthat(...).</p>
    // 
    // 
    // 
    // 
    // @param slice 
    // @param repeat 
    // @return         
    cue { arg slice, repeat; // types Object, Integer          
        if ( repeat == nil , {
            repeat = 1;
        });
        if ( (not (slice.isKindOf(SFSlice))) , {
            slice = this.masterslice.get(slice);
        });
        if ( slice != nil , {
            repeat.do({
                this.cues.add(slice);
            });
        });
        ^this; // explicitly return this, intended use is cl = SFCuelist.new.cue("a").cue("b").cue("c")....
    } // end cue        


    // --- clear() : void -----------------------------------------------------
    //   
    clear {        
        this.cues_(List.new);
    } // end clear        


    // --- play(tempo, verbose) :  --------------------------------------------
    //      
    play { arg tempo, verbose; // types Float, boolean          
        var t;
        var proc;
        if (tempo == nil , { 
            tempo = 1.0; 
        });
        t = (masterslice.master.beats / masterslice.master.length) * tempo;
        proc = BBCuelistProc.new( this );
        proc.verbose = verbose;
        BBCut( BBCutSynthSF(masterslice.master), proc ).tempo_(t).play;
    } // end play        


    // --- post() : void ------------------------------------------------------
    //   
    post {        
        "[".postln;
        cues.do({ arg cue;
            cue.postln;
        });
        "]".post;
    } // end post        


    // --- postln() : void ----------------------------------------------------
    //   
    postln {        
        this.post; "".postln;
    } // end postln        


} // end SFCuelist
