
/* 
 * SuperCollider3 source file "SFSlice.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class SFSlice ----------------------------------------------------------
// 
SFSlice {

    // --- attributes

    var <>name; // type  String
    var <>childrenByName; // type  Dictionary
    var <>ankersByName; // type  Dictionary
    var padCount = 0; // type  int
    var lastSlice; // type  SFSlice


    // --- relationships

    var <>master; // 0..1-relation to type  SF3
    var <>start; // 0..1-relation to type  SFAnker
    var <>end; // 0..1-relation to type  SFAnker
    var <>parent; // 0..1-relation to type  SFSlice
    var children; // 0..*-relation to type  SFSlice
    var ankers; // 0..1-relation to type  SFAnker
    var <>current; // 0..1-relation to type  SFAnker



    // --- new() : SFSlice ----------------------------------------------------
    //     
    *new {        
        ^super.new.ankersByName_(Dictionary.new).childrenByName_(Dictionary.new); // type  SFSlice
    } // end new        


    // --- newMaster(wave) : SFSlice ------------------------------------------
    //      
    *newMaster { arg wave; // type Object         
        ^SFSlice.new.initMaster(wave);
        /*if ( wave.isKindOf(SF3) , {
            ^SFSlice.new.initMaster(wave); // make sure that wave has been set to a high number of beats if it is a long waveform (see below)
        },{ // else: wave is String with file-path
            ^SFSlice.new.initMaster(SF3(wave, 1000)); // type  SFSlice
        });*/
        // note that a high number of beats for the whole file needs to be set
        // in order to make BBCut work at an appropriate granularity on files
        // which are much larger than BBCut had been developed for
    } // end newMaster        


    // --- initMaster(wave) : void --------------------------------------------
    //     
    initMaster { arg wave; // type Object         
        if ( (not (wave.isKindOf(SF3))) , {
            // filename has been passed as string, load sample now
            wave = SF3(wave, 1000); // type  SFSlice            
        });        
        //childrenByName = Dictionary.new;
        //ankersByName = Dictionary.new;
        name = "master";
        master = wave;
        // implicitly created most outer-bounds master ankers
        start = SFAnker.new("master-start", this, nil);
        start.time_( SFTime.new(start).rel(0.0) );
        //"###START: ".post; start.postln;
        end = SFAnker.new("master-end", this, nil).time_( SFTime.new(start).rel(1.0) );
        //current = start;
        current = SFAnker.new("master-current", this, start).time_( SFTime.new(start).rel(0.0) );
    } // end initMaster        


    // --- init(parnt, n, strt, en) : void ------------------------------------
    //        
    init { arg parnt, n, strt, en; // types SFSlice, String, SFAnker, SFAnker            
        //children = Dictionary.new;
        //ankers = Dictionary.new;
        parent = parnt;
        parnt.childrenByName.put(n, this);  // register at parent
        name = n;
        start = strt;
        ankersByName.put("start", strt);
        end = en;
        ankersByName.put("end", en);
        //current = strt.relative.time_(SFTime.new###);
        //current = start;
        current = SFAnker.new("current", this, start).time_( SFTime.new(start).rel(0.0) );
        ankersByName.put("current", current);
        ^this; // type  SFSlice
    } // end init        


    // --- length() : SFTime --------------------------------------------------
    //     
    length {        
        ^end.time.diff(start.time); // type  SFTime
    } // end length        


    // --- slice(name, time) : SFSlice ----------------------------------------
    //
    // <p>Creates a new child-slice of this slice, starting at currentTime and with the length-parameter specified. The length-parameter may be either a double (indicating a relative position in this slice), or a string with a parseable time value.</p>
    // <p>To creates slices which are not directly following each other, insert calls to .pad(length). To create overlapping slices, use .rewind or .windTo and then keep on creating more slices.</p>
    // <p>To create inner slices of a slice, us .inner.</p>
    // 
    // 
    // 
    // 
    // @param name 
    // @param time 
    // @return          
    slice { arg name, time; // types String, Object          
        this.makeSlice(SFSlice.new, name, time);
        ^this; // type  SFSlice (RETURNS _THIS_ to let user create multiple slices of the same parent/master-slice using .slice(...).slice(...).slice(...)...)
    } // end slice        


    // --- pad(length) : SFSlice ----------------------------------------------
    //      
    pad { arg length; // type Object         
        var time;
        padCount = padCount + 1;
        time = SFTime.new(current).parse(length);
        current.time.plus(time);
        ^this; // type  SFSlice
    } // end pad        


    // --- reset() : SFSlice --------------------------------------------------
    //     
    reset {        
        current.time.rel(0);
        ^this; // type  SFSlice
    } // end reset        


    // --- position(pos) : SFSlice --------------------------------------------
    //      
    position { arg pos; // type Object         
        var time;
        time = SFTime.new(this).parse(pos);
        current.time = time;
        ^this; // type  SFSlice
    } // end position        


    // --- inner(i) : SFSlice -------------------------------------------------
    //      
    inner { arg i; // type int         
        if ( i > 0 , {
            ^lastSlice.inner(i-1);
        },{
            ^lastSlice; // type  SFSlice
        });
    } // end inner        


    // --- outer(i) : SFSlice -------------------------------------------------
    //      
    outer { arg i; // type int         
        if ( i > 0 , {
            ^parent.outer(i-1);
        },{
            ^parent; // type  SFSlice
        });
    } // end outer        


    // --- get(name) : SFSlice ------------------------------------------------
    //      
    get { arg name; // type String         
        var ch;
        if ( name == this.name , {
            ^this;
        },{
            ch = childrenByName.at(name);
            if ( ch != nil, {
                ^ch;
            },{
                this.children.do({ arg child;
                    ch = child.get(name);
                    if ( ch != nil , {
                        ^ch;
                    });
                });
            });
            ^nil; // type  SFSlice
        });
    } // end get        


    // --- children() : Array -------------------------------------------------
    //     
    children {        
        // overwrites relationship-getter
        ^childrenByName.values; // type  Array
    } // end children        


    // --- children_(newChildren) : void --------------------------------------
    //     
    children_ { arg newChildren; // type Array         
        // overwrites relationship-setter
        childrenByName.clear;
        newChildren.do({ arg child;
            childrenByName.put(child.name, child);
        });
    } // end children_        


    // --- anker(name, time, relativeTo) : SFSlice ----------------------------
    //        
    anker { arg name, time, relativeTo; // types String, Object, SFAnker           

        // TODO
        ^nil; // type  SFSlice
    } // end anker        


    // --- ankers() : Array ---------------------------------------------------
    //     
    ankers {        
        // overwrites relationship-getter
        ^ankersByName.values; // type  Array
    } // end ankers        


    // --- ankers_(newAnkers) : void ------------------------------------------
    //     
    ankers_ { arg newAnkers; // type Array         
        // overwrites relationship-setter
        ankersByName.clear;
        newAnkers.do({ arg a;
            ankersByName.put(a.name, a);
        });
    } // end ankers_        


    // --- sliceTo(name, endTime) : SFSlice -----------------------------------
    //       
    sliceTo { arg name, endTime; // types String, Object          
        this.makeSliceTo(SFSlice.new, name, endTime);
        ^this; // type  SFSlice (RETURNS _THIS_ to let user create multiple slices of the same parent/master-slice using .slice(...).slice(...).slice(...)...)
    } // end sliceTo        


    // --- sliceRange(name, startTime, endTime) : SFSlice ---------------------
    //        
    sliceRange { arg name, startTime, endTime; // types String, Object, Object           
        this.makeSliceRange(SFSlice.new, name, startTime, endTime);
        ^this; // type  SFSlice (RETURNS _THIS_ to let user create multiple slices of the same parent/master-slice using .slice(...).slice(...).slice(...)...)
    } // end sliceRange        


    // --- masterslice() : SFSlice --------------------------------------------
    //     
    masterslice {        
        //"$$$ MS".postln;
        if ( master != nil , {
            //"$$$ 1".postln;
            ^this;
        },{
            //"$$$ 2".postln;
            if ( parent != nil, {
                ^(parent.masterslice); // type  SFSlice
            },{
                "*** ERROR: no master SF3 object".postln;
                ^nil;
            });
        });
    } // end masterslice        


    // --- post(level) : void -------------------------------------------------
    //     
    post { arg level; // type int         
        if ( level == nil, { 
            level = 0; 
        });
        (Toolbox.repeat("  ", level)).post;
        if ( level > 0 , { "+".post; });
        ("|" ++ name ++ ": ").post; start.post; "-".post; end.post; ", ".post; current.post; " |".post;
        this.children.do({ arg ch;
            "".postln;
            ch.post(level + 1);
        });
    } // end post        


    // --- postln() : void ----------------------------------------------------
    //   
    postln {        
        this.post; "".postln;
    } // end postln        


    // --- padTo(time) : SFSlice ----------------------------------------------
    //      
    padTo { arg time; // type Object         
        // time may be of type Number or String (to specify a time), or SFAnker
        var e;
        if ( time.isKindOf(SFAnker) , {
            e = time.time.relativeTo(this.start);
        },{ // end is of type Number or String: time
            e = SFTime.new(this.start).parse(time);
        });
        current.time.set(e);
    } // end padTo        


    // --- makeSlice(slc, nam, time) :  ---------------------------------------
    //       
    makeSlice { arg slc, nam, time; // types SFSlice, String, Object           
        var l, strt, en;
        //"** makeSlice".postln;
        l = SFTime.new(current).parse(time);
        strt = this.start.relative.name_("start").time_(current.time.clone);
        en = strt.relative.name_("end").time_(current.time.sum(l));
        lastSlice = slc.init(this, nam, strt, en);
        //childrenByName.put(name, lastSlice); (done in slice.init)
        current.time.plus(l);
    } // end makeSlice        


    // --- makeSliceRange(slc, nam, startTime, endTime) :  --------------------
    //        
    makeSliceRange { arg slc, nam, startTime, endTime; // types SFSlice, String, Object, Object            
        // start, end may be of type Number or String (to specify a time), or SFAnker
        var s, e, startAnker, endAnker, t;
        //"** makeSliceRange".postln;        
        if ( startTime.isKindOf(SFAnker) , {
            startAnker = startTime;
        },{ // end is of type Number or String: time
            s = SFTime.new(this.start).parse(startTime);
            startAnker = start.relative.name_("start").time_(s);
        });
        if ( endTime.isKindOf(SFAnker) , {
            endAnker = endTime;
        },{ // end is of type Number or String: time
            e = SFTime.new(this.start).parse(endTime);
            endAnker = start.relative.name_("end").time_(e);
        });
        lastSlice = slc.init(this, nam, startAnker, endAnker);
        //childrenByName.put(name, lastSlice); (done in slice.init)
        //"END-ANKER-TIME: ".post; endAnker.time.postln;
        //"THIS-START: ".post; this.start.postln;
        t = endAnker.time.relativeTo(this.start);
        current.time.set(t);
    } // end makeSliceRange        


    // --- makeSliceTo(slc, nam, endTime) :  ----------------------------------
    //       
    makeSliceTo { arg slc, nam, endTime; // types SFSlice, String, Object           
        // end may be of type Number or String (to specify a time), or SFAnker
        var e, startAnker, endAnker, t;
        //"** makeSliceTo".postln;
        startAnker = this.start.relative.name_("start").time_(current.time.clone);
        if ( endTime.isKindOf(SFAnker) , {
            endAnker = endTime;
        },{ // end is of type Number or String: time
            e = SFTime.new(this.start).parse(endTime);
            endAnker = start.relative.name_("end").time_(e);
        });
        lastSlice = slc.init(this, nam, startAnker, endAnker);
        //childrenByName.put(name, lastSlice); (done in slice.init)
        //"END-ANKER-TIME: ".post; endAnker.time.postln;
        //"THIS-START: ".post; this.start.postln;
        //"E### ".post; endTime.postln;
        //"A### ".post; endAnker.postln;
        //"A### ".post; endAnker.time.postln;
        t = endAnker.time.relativeTo(this.start);
        //"$### ".post; current.postln;
        //"$### ".post; t.postln;
        current.time.set(t);
    } // end makeSliceTo        


    // --- makeSliceRangeLength(slice, name, startTime, lengthTime) : void ----
    //        
    makeSliceRangeLength { arg slice, name, startTime, lengthTime; // types SFSlice, String, Object, Object            
        var l, strt, s, en;
        if ( startTime.isKindOf(SFAnker) , {
            strt = startTime;
        },{ // start is of type Number or String: time
            s = SFTime.new(this.start).parse(startTime);
            strt = start.relative.name_("start").time_(s);
        });
        l = SFTime.new(current).parse(lengthTime);
        //strt = this.start.relative.name_(name++"_start").time_(current.time.clone);
        en = strt.relative.name_("end").time_(strt.time.sum(l));
        lastSlice = slice.init(this, name, strt, en);
        //childrenByName.put(name, lastSlice); (done in slice.init)
        current.time.set(en.time);
    } // end makeSliceRangeLength        


    // --- sliceRangeLength(name, startTime, length) :  -----------------------
    //       
    sliceRangeLength { arg name, startTime, length; // types String, Object, Object           
        makeSliceLength(SFSlice.new, name, startTime, length);
        ^this; // type  SFSlice (RETURNS _THIS_ to let user create multiple slices of the same parent/master-slice using .slice(...).slice(...).slice(...)...)
    } // end sliceRangeLength        


} // end SFSlice
